#! /usr/bin/env python
"""
A part of of this code is taken from one of the demo file of
aubio @ https://github.com/aubio/aubio/blob/master/python/demos/demo_alsa.py

Frequencies of music notes are taken from 
https://pages.mtu.edu/~suits/notefreqs.html to create notefreq.pkl

Thank you!

"""
import numpy as np
import aubio
import sys
if sys.platform == 'linux':
    import alsaaudio
else:
    import pyaudio
sys.path.append('/home/jk/jk/python/')
from mytools import *
import os
import pathlib

def measureFrequency(que):

    # note-freq table - list of two lists
    # [
    #  [f1,    f2,    f3......],
    #  [note1, note2, note3,....]
    # ]
    curdir = pathlib.Path(__file__).parent.resolve()
    freqpath = os.path.join(curdir, 'notefreq.pkl')
    freqnote = load(freqpath)
    #print(freqnote)

    # constants
    samplerate = 44100
    win_s = 2048 * 2 
    hop_s = win_s // 2
    framesize = hop_s
    buffer_size = 1024
    n_channels = 1

    if sys.platform == 'linux':
        # set up audio input
        recorder = alsaaudio.PCM(type=alsaaudio.PCM_CAPTURE)
        recorder.setperiodsize(framesize)
        recorder.setrate(samplerate)
        recorder.setformat(alsaaudio.PCM_FORMAT_FLOAT_LE)
        recorder.setchannels(1)
    else:
        p = pyaudio.PyAudio()
        pyaudio_format = pyaudio.paFloat32
        stream = p.open(format=pyaudio_format,
                channels=n_channels,
                rate=samplerate,
                input=True,
                frames_per_buffer=buffer_size)

    # create aubio pitch detection (first argument is method, "default" is
    # "yinfft", can also be "yin", "mcomb", fcomb", "schmitt").
    #win_s = 4096 # fft size
    #hop_s = buffer_size # hop size
    #hop_s = 1024
    #print('\n*********', win_s, hop_s, '\n***************')
    pitcher = aubio.pitch("default", win_s, hop_s, samplerate)
    # set output unit (can be 'midi', 'cent', 'Hz', ...)
    pitcher.set_unit("Hz")
    # ignore frames under this level (dB)
    pitcher.set_silence(-10)

    print("Starting to listen, press Ctrl+C to stop")

    strings = {
            'e2' : 82,
            'a2' : 110,
            'd3' : 146.83,
            'g3' : 196.00,
            'b3' : 246.94,
            'e4' : 329.63,
            }
    def vInrange(v):
        freqs = freqnote[0] #
        #freqs = strings.values()
        for f in freqs:
            if v < f + 10 and v > f - 10:
                return True
        return False

    tune = 329.63 #(82#110 #146.83 # 196.00#246.94 #329.63)
    print("starts with tuning e4")
    print("to tune a different string presss Ctrl^C.  Then enter\
            the labels e2, a2, d3 etc")
    print("enter q to quit after Ctrl^c")

    count = 0
    freql = []
    avfreq = 0
    lnote = '?'
    hnote = '?'
    # main loop
    while True:
        try:
            if sys.platform == 'linux': # using pyalsaaudio
                # read data from audio input
                _, data = recorder.read()
                # convert data to aubio float samples
                samples = np.fromstring(data, dtype=aubio.float_type)
            else:
                # read data from audio input
                data = stream.read(buffer_size * 2)
                samples = np.fromstring(data, dtype=np.float32)
            # pitch of current frame
            freq = pitcher(samples)[0]
            # compute energy of current block
            energy = np.sum(samples**2)/len(samples)

            #print(freq) if freq != 0 else None

            # eliminating noise by selecting freq around 
            # notes 
            if vInrange(freq): freql.append(freq)
            # finding average freq
            if len(freql) == 2:
                sum_ = 0
                for v in freql:
                        sum_ += v
                        count += 1
                if count != 0:  avfreq = sum_/count
                # checking around which note frequency falls
                # - the lower, middle and upper frequencey for
                # visualization purpose
                # lf, mf, hf
                for i in range(len(freqnote[0]) - 1):
                    if avfreq >= freqnote[0][i] and avfreq <= freqnote[0][i+1]:
                        # delta - separation of notes
                        delta = freqnote[0][i+1] - freqnote[0][i]
                        # if avfreq x<- avfreq ->(halfway)<-     ->y  
                        # then choose a lower freq as lf
                        if avfreq <= freqnote[0][i] + 0.5 * delta:
                            lnote, lf = (freqnote[1][i - 1], freqnote[0][i - 1]) if i!=0 else ('?', 16)
                            mnote, mf = freqnote[1][i], freqnote[0][i]
                            hnote = freqnote[1][i + 1] if i + 1 <= len(freqnote[0]) - 1 else 'C9'
                            hf = freqnote[0][i + 1] if i + 1 <= len(freqnote[0]) - 1 else 8372
                            fmark = int(10 * (avfreq - mf)/(hf - mf))
                            print(lnote + 10 * '~' + mnote + (fmark - 1) * '~' + 'X' + (10 - fmark) * '~' + hnote)
                            # 1 in the last tuple represents the first range
                            # where avfreq falls
                            qdata = ((lf, lnote), (mf, mnote), (hf, hnote), (avfreq, 1))
                            que.put(qdata)
                        # if x<-      (halfway)  avfreq     ->y
                        # then choose next to next freq as hf
                        else:
                            lnote, lf = freqnote[1][i], freqnote[0][i]
                            mnote, mf = freqnote[1][i + 1], freqnote[0][i + 1]
                            hnote = freqnote[1][i + 2] if i + 2 <= len(freqnote[0]) - 1 else 'C9'
                            hf = freqnote[0][i + 2] if i + 2 <= len(freqnote[0]) - 1 else 8372
                            fmark = int(10 * (avfreq - lf)/(mf - lf))
                            print(lnote + (fmark - 1) * '~' + 'X' + (10- fmark) * '~' + mnote + 10 * '~' +  hnote)
                            # 2 in the last tuple represents the second range
                            # where avfreq falls
                            qdata = ((lf, lnote), (mf, mnote), (hf, hnote), (avfreq, 2))
                            que.put(qdata)
                        #print(lnote, mnote, hnote, avfreq)
                        break
                #print(lnote, hnote, avfreq)
                freql = []
                count = 0
                avfreq = 0
        except KeyboardInterrupt:
            print("Ctrl+C pressed, exiting")
            tune_ = input('string: ')
            tune = strings[tune_]
            if tune_ == 'q':
                break
if __name__ == '__main__':
    class QueMock:
        def put(self, x):
            print(x)
    que = QueMock()
    measureFrequency(que)



from tkinter import *
import sys, os
import numpy as np
from tkinter import *
from pdb import set_trace as tr
import time
import random
from queue import Queue, Empty
from threading import Thread
import pathlib

if sys.platform == 'linux':
    from guitartuner._guitartuner import measureFrequency
else:
    from guitartuner._guitartuner_windows import measureFrequency

def startTuner():

    width = 300
    height = width//2
    margin = 5
    centerx, centery = width//2, height - 2 * margin
    r = width - margin
    r = 100
    thetas = 0; thetae = 180 # start, stop
    N = 23 # 1 + 10 + 1 + 10 + 1
    thetapieces = np.linspace(thetas, thetae, N) * np.pi / 180 # in radians
    # 10 divisions on each side 
    # including 3 big ones - 10 + 10 + 3
    x = r * np.cos(thetapieces)
    y = r * np.sin(thetapieces)
    # correcting with center co-ordinates
    x = centerx + x
    y = centery - y
    root = Tk()
    root.title('Guitar Tuner')
    #iconpath = os.path.abspath('iconguitar.png')
    curdir = pathlib.Path(__file__).parent.resolve()
    iconpath = os.path.join(curdir, 'iconguitar.png')
    icon = PhotoImage(file=iconpath)
    root.iconphoto(False, icon)
    root.geometry(str(width+50)+'x'+str(height+50))
    can = Canvas(root, width=width, height=height)
    can.pack(side=TOP)
    Button(root, text='Quit', command=sys.exit).pack(side=TOP)
    thetamains = np.array([0, 90, 180]) * np.pi/180
    # 0, 90 , 180 @ 0, 1, 2
    X = r * np.cos(thetamains)
    Y = r * np.sin(thetamains)
    X = centerx + X; Y = centery - Y
    que = Queue()
    def producer():
        #while True:
        #    time.sleep(1)
        #    que.put(random.choice(range(len(x))))
        measureFrequency(que)
    Thread(target=producer, daemon=True).start()
    # last data initialization
    prevdata = [(1, '-'), (2, '-'), (4, '-'), (4, 2)]
    def drawtuner(endx, endy):
        nonlocal prevdata
        can.delete('all')
        for i, j in zip(x, y):
            can.create_oval(i, j, i+2, j +2)
        # three notes - thetas 0, 90, 180
        for i, j in zip(X, Y):
            can.create_oval(i-4, j-4, i+5, j +5)
        # center circle
        can.create_oval(centerx-1, centery-1, centerx+1, centery+1)
        # needle
        can.create_line(centerx, centery, endx, endy, arrow=LAST)
        try:
            data = que.get(block=False)
            prevdata = data
            #print('data: -->', data)
        except Empty:
            #data = [(1, '-'), (2, '-'), (4, '-'), (4, 2)]
            data = prevdata
        # data extraction
        lf, lnote = data[0][0], data[0][1]
        mf, mnote = data[1][0], data[1][1]
        hf, hnote = data[2][0], data[2][1]
        avfreq, drange = data[3][0], data[3][1]
        # data conversion - to canvas - logic
        if data[3][1] == 1: # avfreq in 180-90
            thetaf = 180 + ((90 - 180)/(mf - lf)) * (avfreq - lf)
            thetaf *= np.pi/180  # in radians
        else: 
            thetaf = 90 + ((0 - 90)/(hf - mf)) * (avfreq - mf)
            thetaf *= np.pi/180 # in radians
        # for out of range values
        #if thetaf > np.pi/2: thetaf = 0
        # need head for avfreq
        xf = r * np.cos(thetaf)
        yf = r * np.sin(thetaf)
        # correcting for center
        xf = centerx + xf
        yf = centery - yf
        # note labels in reverse order
        can.create_text(X[0] + 25, Y[0], text=data[2][1])
        can.create_text(X[1], Y[1]-25, text=data[1][1])
        can.create_text(X[2] - 25, Y[2], text=data[0][1])
        choice = 0
        #print(data)
        can.after(10, lambda : drawtuner(xf, yf))
    drawtuner(X[0], Y[0])
    mainloop()

if __name__ == '__main__':
    startTuner()
